﻿using System;
using System.Collections.ObjectModel;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace Desktop_LabelEdge
{
    public partial class MainForm : Form
    {
        private ShapeFileFeatureLayer shapeFileFeatureLayer;

        public MainForm()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {

            // Part one ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ We directly converts the ShapeFileFeatureLayer to InMemoryFeatureLayer and modify polygon, it is good for small shapefiles.
            winformsMap1.MapUnit = GeographyUnit.Meter;
            // Create shapeFileFeatureLayer
            string shapefileName = @"../../data/TestLayer.shp";
            shapeFileFeatureLayer = new ShapeFileFeatureLayer(shapefileName);
            // Add Projection.
            ManagedProj4Projection oProj4 = new ManagedProj4Projection(ManagedProj4Projection.GetEsriParametersString(102003), ManagedProj4Projection.GetGoogleMapParametersString());
            shapeFileFeatureLayer.FeatureSource.Projection = oProj4;

            // Create text style.
            var oAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Blue);
            var oTextStyle = new TextStyle("StandNumbe", new GeoFont("Arial", 10, DrawingFontStyles.Regular), new GeoSolidBrush(GeoColor.SimpleColors.Black));
            oTextStyle.FittingPolygonInScreen = true;

            // Create an InMemoryFeatureLayer by shapFileFeatureLayer.
            InMemoryFeatureLayer resultLayer = GetBufferLayer(shapeFileFeatureLayer);
            // Add text style to shapeFileFeatureLayer.
            resultLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = oAreaStyle;
            resultLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = oTextStyle;
            resultLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay layerOverlay = new LayerOverlay();
            layerOverlay.Layers.Add(resultLayer);
            winformsMap1.Overlays.Add(layerOverlay);

            shapeFileFeatureLayer.Open();
            winformsMap1.CurrentExtent = shapeFileFeatureLayer.GetBoundingBox();
            shapeFileFeatureLayer.Close();
            winformsMap1.Refresh();


            ////// Part two ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ We add a event to modify DrawingFeatures. it is suitable for large data.
            //winformsMap1.MapUnit = GeographyUnit.Meter;
            //// Create shapeFileFeatureLayer
            //string shapefileName = @"../../data/TestLayer.shp";
            //shapeFileFeatureLayer = new ShapeFileFeatureLayer(shapefileName);
            //// Add Projection.
            //ManagedProj4Projection oProj4 = new ManagedProj4Projection(ManagedProj4Projection.GetEsriParametersString(102003), ManagedProj4Projection.GetGoogleMapParametersString());
            //shapeFileFeatureLayer.FeatureSource.Projection = oProj4;

            //// Add a event to modify polygon.
            //shapeFileFeatureLayer.DrawingFeatures += ShapeFileFeatureLayer_DrawingFeatures;

            //// Create text style.
            //var oAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.SimpleColors.Transparent, GeoColor.SimpleColors.Blue);
            //var oTextStyle = new TextStyle("StandNumbe", new GeoFont("Arial", 10, DrawingFontStyles.Regular), new GeoSolidBrush(GeoColor.SimpleColors.Black));
            //oTextStyle.FittingPolygonInScreen = true;

            //// Add text style to shapeFileFeatureLayer.
            //shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = oAreaStyle;
            //shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = oTextStyle;
            //shapeFileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //LayerOverlay layerOverlay = new LayerOverlay();
            //layerOverlay.Layers.Add(shapeFileFeatureLayer);
            //winformsMap1.Overlays.Add(layerOverlay);

            //shapeFileFeatureLayer.Open();
            //winformsMap1.CurrentExtent = shapeFileFeatureLayer.GetBoundingBox();
            //shapeFileFeatureLayer.Close();
            //winformsMap1.Refresh();
        }

        private void ShapeFileFeatureLayer_DrawingFeatures(object sender, DrawingFeaturesEventArgs e)
        {
            Collection<Feature> tempFeatures = new Collection<Feature>(); ;
            foreach (Feature feature in e.FeaturesToDraw)
            {
                var bufferedFeature = feature.Buffer(0, winformsMap1.MapUnit, DistanceUnit.Meter);
                tempFeatures.Add(bufferedFeature);
            }
            e.FeaturesToDraw.Clear();
            foreach (Feature feature in tempFeatures)
            {
                e.FeaturesToDraw.Add(feature);
            }
        }

        private InMemoryFeatureLayer GetBufferLayer(ShapeFileFeatureLayer shapeFileFeatureLayer)
        {
            shapeFileFeatureLayer.Open();

            var featureSourceColumns= shapeFileFeatureLayer.FeatureSource.GetColumns();
            InMemoryFeatureLayer returenInmemoryFeature = new InMemoryFeatureLayer(featureSourceColumns);

            var allFeatures = shapeFileFeatureLayer.FeatureSource.GetAllFeatures( ReturningColumnsType.AllColumns);

            var targetBoundingBox = shapeFileFeatureLayer.GetBoundingBox().GetFeature();

            foreach (Feature featuer in allFeatures)
            {
                var bufferedFeature = featuer.Buffer(0,winformsMap1.MapUnit, DistanceUnit.Meter);
                returenInmemoryFeature.InternalFeatures.Add(bufferedFeature);
            }
            
            return returenInmemoryFeature;
        }
    }
}
